/** Class to help installing BSF4Rexx, will output all <code>java.lang.System.getProperties()</code>
 *  as a &quot;<code>propertyName=[value]</code>&quote; pair, each delimited with
 *  &quot;<code>\\n</code>&quot;. This allows RxQueue'ing the output into any Rexx program for further
 *  analysis.
 *
 * <p>
 *
 * <pre>------------------------ Apache Version 2.0 license -------------------------
 *    Copyright (C) 2005 Rony G. Flatscher
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 * ----------------------------------------------------------------------------- </pre>
 *
 *
 * @author Rony G. Flatscher (c) 2005, 2006
 * @version 1.0.0
 *
 */

 /*
    changes:
        2006-08-01: now tests for presence of "org.apache.commons.logging.LogFactory"; returning
                    "rgf.commons.logging.available=[1]" if loadable, else
                    "rgf.commons.logging.available=[0]"

   last change: $Revision: 168 $ $Author: rony $ $Date: 2006-08-01 21:42:20 +0200 (Tue, 01 Aug 2006) $

 */

public class JavaInfo4BSF4RexxInstallation
{
    public static void main (String args[])
    {
        // showSomeProperties();
        showAllProperties();
        System.exit(0);
    }

    static char [] letters={ '0', '1', '2', '3', '4', '5', '6', '7',
                             '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };

    static void showSomeProperties()
    {
        StringBuffer sb=new StringBuffer();    // create a StringBuffer
        String       nl=System.getProperty("line.separator");

        sb=sb.append("file.separator="             ).append("[").append(  nullString(System.getProperty("file.separator"             ))).append("]").append(nl);
        sb=sb.append("path.separator="             ).append("[").append(  nullString(System.getProperty("path.separator"             ))).append("]").append(nl);
        sb=sb.append("line.separator="             ).append("[").append(  nullString(System.getProperty("line.separator"             ))).append("]").append(nl);

        sb=sb.append("java.class.version="         ).append("[").append(  nullString(System.getProperty("java.class.version"         ))).append("]").append(nl);
        sb=sb.append("java.ext.dirs="              ).append("[").append(  nullString(System.getProperty("java.ext.dirs"              ))).append("]").append(nl);
        sb=sb.append("java.home="                  ).append("[").append(  nullString(System.getProperty("java.home"                  ))).append("]").append(nl);
        sb=sb.append("java.specification.version=" ).append("[").append(  nullString(System.getProperty("java.specification.version" ))).append("]").append(nl);
        sb=sb.append("java.vendor.url"             ).append("[").append(  nullString(System.getProperty("java.vendor.url"            ))).append("]").append(nl);
        sb=sb.append("java.vendor="                ).append("[").append(  nullString(System.getProperty("java.vendor"                ))).append("]").append(nl);
        sb=sb.append("java.version="               ).append("[").append(  nullString(System.getProperty("java.version"               ))).append("]").append(nl);

        sb=sb.append("os.arch="                    ).append("[").append(  nullString(System.getProperty("os.arch"                    ))).append("]").append(nl);
        sb=sb.append("os.name="                    ).append("[").append(  nullString(System.getProperty("os.name"                    ))).append("]").append(nl);
        sb=sb.append("os.version="                 ).append("[").append(  nullString(System.getProperty("os.version"                 ))).append("]").append(nl);

        sb=sb.append("user.name="                  ).append("[").append(  nullString(System.getProperty("user.name"                  ))).append("]").append(nl);
        sb=sb.append("user.home="                  ).append("[").append(  nullString(System.getProperty("user.home"                  ))).append("]").append(nl);
        sb=sb.append("user.dir="                   ).append("[").append(  nullString(System.getProperty("user.dir"                   ))).append("]").append(nl);

        sb=sb.append("java.library.path="          ).append("[").append(  nullString(System.getProperty("java.library.path"          ))).append("]").append(nl);
        sb=sb.append("java.class.path="            ).append("[").append(  nullString(System.getProperty("java.class.path"            ))).append("]").append(nl);

            // test for availability of org.apache.commons.logging
        sb=sb.append( get_commons_logging_availability()).append(nl);

        System.out.println(sb);
    }

    static void showAllProperties()
    {
        java.util.Enumeration prop_names=System.getProperties().propertyNames();   // get all the property names

        StringBuffer sb=new StringBuffer();    // create a StringBuffer
        String       nl=System.getProperty("line.separator");
        String       key="";

        while (prop_names.hasMoreElements())
        {
            key=(String) prop_names.nextElement();
            sb=sb.append(key).append("=[").append( nullString(System.getProperty(key))).append("]").append(nl);;
        }

            // test for availability of org.apache.commons.logging
        sb=sb.append( get_commons_logging_availability()).append(nl);

        System.out.println(sb);
    }

    static String nullString(String o)      // if null, return empty String instead
    {
        if (o==null) return "";

        // check for chars < x20, encode them
        StringBuffer sb=new StringBuffer();
        char         ch=0;
        for (int i=0; i<o.length(); i++)
        {
            if (o.charAt(i)>31)
               continue;
            // o.k., char < 0x20: we need to transform this string into a hex-string "0xHEXDATA"
            sb.append("0x");
            for (int k=0; k<o.length(); k++)
            {
                ch=o.charAt(k);
                sb.append(letters[(ch >>> 16)]);
                sb.append(letters[(ch & 0x0f) ]);
            }
            return sb.toString();
        }

        return o;
    }



    static String get_commons_logging_availability()
    {
        String kw="rgf.commons.logging.available=[";    // define keyword and leadin
        try
        {
            // try to load the class dynamically
            Class lf=Class.forName("org.apache.commons.logging.LogFactory");
        }
        catch (ClassNotFoundException e)
        {
            return kw+"0]"; // return Rexx character for .false
        }
        return kw+"1]";     // return Rexx character for .true

    }

}
