/*
   Name:    setupOOo.rex
   Purpose: create a shell script which sets the environment variable CLASSPATH
            to point to the Java archives needed to drive/automate OpenOffice.org
            resp. StarOffice
   Author:  Rony G. Flatscher
   Version: 1.06
   Date:    2006-01-04
   Changed: 2006-01-16, ---rgf, make sure CLASSPATH is used in any case
            2006-01-22, ---rgf, on Linux hint the user how to enter "setEnvironment4BSF4Rexx.sh"
            2006-01-24, ---rgf, now OOo-Reflection support unbundled, needs to be copied to
                                "OOo/program/classes" and then put into CLASSPATH
            2006-02-25, ---rgf, removed a quirk in a Unix string (delimiter part of string!),
                                "setEnvironment4OOo.*" is now copied to a PATH directory

   last change: $Revision: 165 $ $Author: rony $ $Date: 2006-07-17 00:14:07 +0200 (Mon, 17 Jul 2006) $

   license:

    ------------------------ Apache Version 2.0 license -------------------------
       Copyright (C) 2006 Rony G. Flatscher

       Licensed under the Apache License, Version 2.0 (the "License");
       you may not use this file except in compliance with the License.
       You may obtain a copy of the License at

           http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing, software
       distributed under the License is distributed on an "AS IS" BASIS,
       WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
       See the License for the specific language governing permissions and
       limitations under the License.
    -----------------------------------------------------------------------------
*/

cmd.=""                          /* default value for stem        */
parse upper source s +1          /* get first character of operating system in uppercase */
cmd.eIsUnix=(pos(s, "WO")=0)

/* for debugging: force Unix-mode
   cmd.eIsUnix=1
*/

if cmd.eIsUnix=0 then
do
   cmd.eIsUnix=0                 /* Windows/OS2 style       */
   cmd.eFile.sep="\"
   cmd.ePath.sep=";"
end
else
do
   cmd.eIsUnix=1                 /* Unix style              */
   cmd.eFile.sep="/"
   cmd.ePath.sep=":"
end

                                 /* paths to Java and desired bin-dir given?  */
parse arg path2OOo               /* get path to OpenOffice.org/StarOffice     */
oldDir=directory()               /* memorize actual directory                 */
cmd.ePath2OOo=directory(strip(path2OOo, 'B', '"') || cmd.eFile.sep || "program" || cmd.eFile.sep || "classes")

call directory oldDir            /* reset directory                           */
if cmd.ePath2OOo="" then         /* not found, give a message and abort ...   */
do
   say "path to OpenOffice.org/StarOffice directory" pp(path2OOo) "could not be found!"
   say
   say "     usage: rexx setupOOo path-to-OpenOffice-directory"
   say
   if cmd.eIsUnix=0 then
      say '   example: rexx setupOOo d:\Programme\OpenOffice.org 2.0'
   else
      say '   example: rexx setupOOo /opt/openoffice.org.2.0'
   exit -1
end

/* for forcing testing Unix on Windows
j.cmd.eFile.separator="/" -- rgf, debug
j.cmd.ePath.separator=":" -- rgf, debug
*/

   /* parse path, put directories into the stem variable */
path.=""
call parsePath

cp=value("CLASSPATH", , "ENVIRONMENT") /* try to get the CLASSPATH value   */
cmd.eRgfReflectUNO="oorexx-uno.jar"
cmd.eSupportInfos=qc("Support: Internet newsgroup <news:comp.lang.rexx>, <http://www.OpenOffice.org> or via <http://www.RexxLA.org>")

if cmd.eIsUnix then
do
   nl="0a"x                      /* Unix new-line character sequence (\n)  */
   cmd.eLeadIn="#!/bin/sh"nl     /* command to execute this file           */
   cmd.eEcho="echo"
   cmd.eCopy="cp -plfv"          /* preserve, hard-link, force, verbose: source target(dir)  */
   cmd.eDelete="rm -fv"
   cmd.eSetOOo = "OOoJar=" ||cmd.ePath2OOo
   cmd.eJarVar = cmd.ePath.sep"$OOoJar"cmd.eFile.sep   /* variable to use                        */
   if cp="" then
      cmd.eSetCP  = "CLASSPATH=."            /* make sure local dir is searched!       */
   else
      cmd.eSetCP  = "CLASSPATH=$CLASSPATH"

   cmd.eSetCP2 = "export CLASSPATH"

   cmd.eInstallScriptName="installOOo.sh"
   cmd.eUninstallScriptName="uninstallOOo.sh"

   if cmd.eBinDir="" then
      cmd.eBinDir="/usr/bin"     /* directory where executables reside     */

   cmd.eSetEnvironmentScriptName="setEnvironment4OOo.sh"
   cmd.eExecuteEnvironmentScript=". "cmd.eSetEnvironmentScriptName
   cmd.eSetExecutable    ="chmod +755"   /* set executable for all (user, group, other)  */
   cmd.eLineComment="#"
end
else  /* Windows, OS/2 (eCS), ... */
do
   nl="0d0a"x                    /* Windows/OS2 new-line character sequence (\n) */
   cmd.eEcho="@echo"
   cmd.eCopy="copy /y"
   cmd.eDelete="del /f"
   cmd.eSetOOo = 'set OOoJar='cmd.ePath2OOo  /* enquote in case path contains a blank  */
   cmd.eJarVar = cmd.ePath.sep"%OOoJar%"cmd.eFile.sep  /* variable to use                        */
   if cp="" then
      cmd.eSetCP  = "set CLASSPATH=."        /* make sure local dir is searched!       */
   else
      cmd.eSetCP  = "set CLASSPATH=%CLASSPATH%"

   cmd.eInstallScriptName="installOOo.cmd"
   cmd.eUninstallScriptName="uninstallOOo.cmd"

   if cmd.eBinDir="" then
      cmd.eBinDir=getRexxDir()   /* directory where executables reside     */

   cmd.eSetEnvironmentScriptName="setEnvironment4OOo.cmd"
   cmd.eExecuteEnvironmentScript="  " cmd.eSetEnvironmentScriptName
   cmd.eLineComment="@REM "
end

jars="jurt.jar unoil.jar ridl.jar juh.jar" cmd.eRgfReflectUNO,
     "sandbox.jar"               /* only needed for OOo < 2.0              */
do i=1 to words(jars)            /* loop over words                        */
   cmd.eSetCP=cmd.eSetCP || cmd.eJarVar || word(jars, i) /* extract word and append it */
end

/* make sure that CLASSPATH is used in case this script runs at a time, when other programs have set
   the classpath environment variable
*/
if cp="" then
do
  if cmd.eIsUnix=1 then
     cmd.eSetCP=cmd.eSetCP || cmd.ePath.sep || "$CLASSPATH"
  else
     cmd.eSetCP=cmd.eSetCP || cmd.ePath.sep || "%CLASSPATH%"
end

   /* create the scripts   */
call createScript
call createInstallScript

call info         /* tell the user what to do with it */

exit


/* create the install script: this will copy the BSF4Rexx jar-file and dynamic link library
   to the appropriate Java extension directory
*/
createInstallScript: procedure expose j. cmd. path.
  fn=cmd.eInstallScriptName            /* name of install script              */
  ui=cmd.eUninstallScriptName          /* name of uninstall script            */

  call stream fn, "c", "open replace"
  call stream ui, "c", "open replace"

  call lineout fn, cmd.eLeadIn
  call lineout fn, cmd.eEcho qe("created:" q(date("S") time()))
  call lineout fn, cmd.eEcho qe("installing ""OpenOffice.org"" support...")
  call lineout fn, ""

     /* uninstall: */
  call lineout ui, cmd.eLeadIn
  call lineout ui, cmd.eEcho qe("created:" q(date("S") time()))
  call lineout ui, cmd.eEcho qe("uninstalling ""OpenOffice.org"" support...")
  call lineout ui, ""

  -- rgf, 2006-07-11: add package to OOo via Rexx and the now established OOo-support
  call lineout ui, cmd.eEcho qe("Removing package ScriptProviderForooRexx.jar from OpenOffice/StarOffice:")
  if cmd.eIsUnix=1 then
     call lineout ui, "."    cmd.eSetEnvironmentScriptName ">/dev/null;rexx add_ooRexxPackage.rex remove >/dev/null"
  else
     call lineout ui, "call" cmd.eSetEnvironmentScriptName ">nul && rexx add_ooRexxPackage.rex remove 2>nul"

  call lineout ui, ""

     /* copying jar-file to OOo classes-directory */
  tgtDir=cmd.ePath2OOo
  tgtFile=stream(cmd.eRgfReflectUNO, "C","Query EXISTS")

  call lineout fn, cmd.eEcho qe("copying UNO reflection jar" q(cmd.eRgfReflectUNO) "to" q(tgtDir) "...")
  call lineout fn, cmd.eCopy qw(cmd.eRgfReflectUNO) qw(tgtDir)
  call lineout fn, ""

  call lineout ui, cmd.eEcho qe("deleting" q(cmd.eRgfReflectUNO)  "from directory" q(tgtDir) "...")
  call lineout ui, cmd.eDelete qw(tgtDir || cmd.eFile.sep || cmd.eRgfReflectUNO)
  call lineout ui, ""


   /* copying setEnvironment4* to a PATH-directory */
  tgtDir=cmd.eBinDir
  -- tgtFile=stream(cmd.eSetEnvironmentScriptName, "C","Query EXISTS")
  tgtFile=cmd.eSetEnvironmentScriptName
  call lineout fn, cmd.eEcho qe("copying"  q(tgtFile) "to" q(tgtDir) "...")
  call lineout fn, cmd.eCopy qw(tgtFile) qw(tgtDir)
  call lineout fn, ""

  call lineout ui, cmd.eEcho qe("deleting" q(tgtFile)  "from directory" q(tgtDir) "...")
  call lineout ui, cmd.eDelete qw(tgtDir || cmd.eFile.sep || cmd.eSetEnvironmentScriptName)
  call lineout ui, ""

  call lineout fn, cmd.eEcho qe("If addressing OOo from the command line, then setup the environment beforehand using:")
  call lineout fn, ""
  call lineout fn, cmd.eEcho qe("   " cmd.eExecuteEnvironmentScript)

  call lineout fn, ""
  call lineout fn, cmd.eEcho qe("done.")
  call lineout fn, ""
  call lineout fn, cmd.eEcho cmd.eSupportInfos
  call lineout fn, ""

  call lineout ui, ""
  call lineout ui, cmd.eEcho qe("done.")
  call lineout ui, ""
  call lineout ui, cmd.eEcho cmd.eSupportInfos
  call lineout ui, ""

  -- rgf, 2006-07-11: add package to OOo via Rexx and the now established OOo-support
  call lineout fn, cmd.eEcho qe("Adding package ScriptProviderForooRexx.jar to OpenOffice/StarOffice:")
  if cmd.eIsUnix=1 then
     call lineout fn, "."    cmd.eSetEnvironmentScriptName ">/dev/null;rexx add_ooRexxPackage.rex >/dev/null"
  else
     call lineout fn, "call" cmd.eSetEnvironmentScriptName ">nul && rexx add_ooRexxPackage.rex 2>nul"

  call lineout fn, cmd.eEcho qe("done.")
  call lineout fn, ""
  call lineout fn, cmd.eEcho qe("/// Please close ALL instances of OpenOffice/StarOffice and the quickstarter.")
  call lineout fn, cmd.eEcho qe("(This will allow OpenOffice/StarOffice to fully initialize and find ooRexx!)")
  call lineout fn, ""
  call lineout ui, cmd.eEcho qe("finished.")


  call stream fn, "c", "close"
  call stream ui, "c", "close"

  if cmd.eSetExecutable<>"" then       /* set file executable (Unix)    */
  do
     cmd.eSetExecutable cmd.eInstallScriptName
     cmd.eSetExecutable cmd.eUninstallScriptName
  end
  return




/* create the test script which sets up all needed parameters to run Rexx programs from
   this directory */
createScript: procedure expose cmd. path.
  fn=cmd.eSetEnvironmentScriptName

  call stream fn, "c", "open replace"
  call lineout fn, cmd.eLeadIn
  call lineout fn, ""
  call lineout fn, cmd.eLineComment "created:" q(date("S") time())
  call lineout fn, cmd.eEcho qe("setting the CLASSPATH environment variable to point to the OpenOffice.org/StarOffice jar files ...")
  call lineout fn, cmd.eLineComment "just run it, before invoking a Rexx program that addresses OpenOffice.org/StarOffice ..."
  call lineout fn, ""

  call lineout fn, cmd.eLineComment "set OOo/SO directory that contains the needed jars..."
  call lineout fn, cmd.eSetOOo
  call lineout fn, ""
  call lineout fn, cmd.eLineComment "copy UNO-reflection jar to OOo/SO program/classes directory..."
  call lineout fn, cmd.eCopy qw(stream(cmd.eRgfReflectUNO, "C","Query EXISTS")) qw(cmd.ePath2OOo)
  call lineout fn, ""

  call lineout fn, cmd.eLineComment "set CLASSPATH environment variable ..."
  call lineout fn, cmd.eLineComment "please note: if using OOo/SO version 2.0 or greater, you do not need ""sandbox.jar"""
  call lineout fn, cmd.eSetCP          /* set CLASSPATH        */
  call lineout fn, cmd.eSetCP2         /* export it on Unix    */
  call lineout fn, ""

  call lineout fn, cmd.eEcho qe(".")
  call lineout fn, cmd.eEcho qe("It is advised to add this environment setting to your systems profile!")

  call lineout fn, cmd.eEcho qe("(This way it is always assured, that Rexx can find the OOo Java classes.)")

  if cmd.eIsUnix=1 then
  do
     call lineout fn, cmd.eEcho qe("E.g. add the following entries to your '~/.bashrc':")
     call lineout fn, cmd.eEcho qe(" ")
     call lineout fn, cmd.eEcho qe("   # allow using OpenOffice.org via its Java interface classes")
     call lineout fn, cmd.eEcho qe("   ." cmd.eSetEnvironmentScriptName ">/dev/null")
  end

  call lineout fn, ""


  call stream fn, "c", "close"

  if cmd.eSetExecutable<>"" then       /* set file executable (Unix)    */
     cmd.eSetExecutable cmd.eSetEnvironmentScriptName

  return



pp: procedure                    /* poor man's "pretty" print     */
  return "[" || arg(1) || "]"


q: procedure                     /* enclose string into apostrophes  */
  return """" || arg(1) || """"

   /* "qe": quote echo string: enquote entire string on Unix only */
qe: procedure expose j. cmd. path.
  if  cmd.eIsUnix=1 then return "'" || arg(1) || "'"
                    else return arg(1)


   /* "qe": quote fully-qualified paths in Windows as they may contain a blank */
qw: procedure expose j. cmd. path.
  if  cmd.eIsUnix=0 then return '"' || arg(1) || '"'
                    else return arg(1)


   /* "qc": quote string in apostrophes, if Unix, in quotes, if Windows (for support infos that contain <, >) */
qc: procedure expose j. cmd. path.
  if  cmd.eIsUnix=0 then return '"' || arg(1) || '"'     /* Windows  */
                    else return "'"  || arg(1) || "'"    /* Unix     */


iif: procedure                   /* VBasic-like function          */
  if arg(1)=1 then return arg(2)
  return arg(3)


info:                            /* tell user what has happened   */
  say
  say "the following scripts have been created:"
  say
  say " " pp(cmd.eInstallScriptName)": install ooRexx-UNO support for OpenOffice"
  if cmd.eIsUnix=0 then
     say "       usage:" cmd.eInstallScriptName
  else
     say "       usage:" q("./"cmd.eInstallScriptName)
  say

  say " " pp(cmd.eUninstallScriptName)": uninstall ooRexx-UNO support for OpenOffice"
  if cmd.eIsUnix=0 then
     say "       usage:" cmd.eUninstallScriptName
  else
     say "       usage:" q("./"cmd.eUninstallScriptName)
  say
  say " " pp(cmd.eSetEnvironmentScriptName)": use to set the environment variable 'CLASSPATH' to"
  say "              point to the OpenOffice.org (OOo)/StarOffice (SO) jar-files"
  say
  if cmd.eIsUnix=0 then
     say "       usage:" cmd.eSetEnvironmentScriptName
  else
     say "       usage:" q(". ./"cmd.eSetEnvironmentScriptName)

  say
  say "     remarks: - After running this script you may invoke any BSF4Rexx-program"
  say "                automating OOo/SO via BSF4Rexx', e.g. (try it!)"
  say
  if cmd.eIsUnix=0 then
     say "                  " cmd.eSetEnvironmentScriptName
  else
     say "                  " q(". ./"cmd.eSetEnvironmentScriptName)

  say "                  " "rexx testOOo.rex"
  say
  say "Good luck! :-)  [You may also want to take a look at this generated script.]"
  exit 0

/* Get path to Rexx or Regina from PATH environment variable and return it;
   Windows/OS2
*/
getRexxDir: procedure expose j. cmd. path.
     /* use this precedence order to locate a path where to copy Rexx-files to  */
  dir=findPath("\ooREXX \regina rexx")
  if dir<>"" then return dir

  eyeCatcher="?!no-rexx-path-found!?"
  say "PROBLEM: could not find explicit path to a Rexx-directory!"
  say "         Please adjust the created scripts to point to any one directory listed"
  say "         with the environment variable ""PATH"", ie. replace" pp(eyecatcher) "with"
  say "         any of the listed PATH directory, and then run the scripts."
  return eyecatcher              /* indicate that not found       */



/* parse path into stem path. */
parsePath: procedure expose j. cmd. path.
/*
say "... j.path.separator=["j.path.separator"]"
say "... j.file.separator=["j.file.separator"]"
*/

  p=value("PATH", , "ENVIRONMENT")
  i=0
  do while p<>""
     parse var p chunk (cmd.ePath.sep) p
     i=i+1

     if cmd.eIsUnix=1 then
        path.i=chunk
     else
        path.i=changestr('"', chunk, "")  /* remove any quotes from any part of the Windows path */

     path.i.upperCase=translate(chunk)  /* uppercase version */
/*
say "... i="i "chunk="pp(chunk)
*/

     path.0=i
  end

  return

/* find needle in path and return it   */
findPath: procedure expose j. cmd. path.
   parse upper arg needles    /* blank delimited, translated into uppercase   */
   do i=1 to words(needles)   /* iterate over needles                         */
      n=word(needles, i)
      do k=1 to path.0        /* iterate over paths      */
         if pos(word(needles,i), path.k.upperCase)>0 then
            return path.k
      end
   end
   return ""

